//////////////////////////////////////////
// EIOTest.cpp
// 710020-11365
//
// History
// =======
// -11364 
// Feb 25, 03 : ctacke
//   Initial release
//
// Feb 26, 03 : ctacke
//   - updated test pattern from a simple count

// Description
// -----------
// This application demonstrates the use of the 
// Extended IO (EIO) WinCE Port of the ADS BitsyPlus.
// The EIO can be controlled bitwise, each having
// a state (1=high, 0=low) and a direction (1=output,
// 0=input).  This application opens the port, sets 
// half of the bits as inputs, half as outputs, then 
// writes a test pattern across all lines.
//////////////////////////////////////////

#include <windows.h>
#include "eio_ctrl.h"

#define EIO_INPUT	0
#define EIO_OUTPUT	1

/***** global variables ****/
// EIO lines         9 8 7 6 5 4 3 2 1 0
// direction mask    0 0 0 0 0 1 1 1 1 1 (0x001F)
// initial state     1 1 1 1 1 1 1 1 1 1 (0x03FF)

static   DWORD	dirmask	= 0x001F;
static	 DWORD	state	= 0x03FF;
volatile BOOL	bRun	= TRUE;

/***** function prototypes *****/
BOOL  EIOSetDirectionMask(HANDLE hEIO, DWORD mask);
BOOL  EIOGetDirectionMask(HANDLE hEIO, DWORD *mask);
BOOL  EIOWrite(HANDLE hEIO, DWORD state);
BOOL  EIORead(HANDLE hEIO, DWORD *state);

ULONG TestThread(void *arg);
void ToBinary(DWORD number, TCHAR *result);

/************************* WinMain Method *************************/
int WINAPI WinMain(
    HINSTANCE   hInst,
    HINSTANCE   hInstPrev,
    LPTSTR      lpszCmdLine,
    int         nCmdShow)
{
	HANDLE	hEIO		= NULL;
	DWORD	threadID	= 0;
	HANDLE	hThread		= NULL;

	RETAILMSG(TRUE, (_T("-------- Begin EIOTest --------\r\n")));
	RETAILMSG(TRUE, (_T("  Opening EIO1: Port...")));

	// first get a handle for the EIO driver
	hEIO = CreateFile(TEXT("EIO1:"), GENERIC_READ | GENERIC_WRITE, FILE_SHARE_READ,
		NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);

	// check for success
	if (!hEIO) 
	{
		RETAILMSG(TRUE, (_T("failed (%i)\r\n"), GetLastError()));
		return -1;
	}

	RETAILMSG(TRUE, (_T("  ok.\r\n")));

	// configure the EIO lines.  0-4 are output, 5-9 are inputs
	RETAILMSG(TRUE, (_T("  setting direction mask to 0x%x..."), dirmask));
	if(!EIOSetDirectionMask(hEIO, dirmask))
	{
		RETAILMSG(TRUE, (_T("failed (%i)\r\n"), GetLastError()));
		return -1;
	}
	RETAILMSG(TRUE, (_T("  ok.\r\n")));

	// read back the mask
	RETAILMSG(TRUE, (_T("  reading back mask...")));
	if(!EIOGetDirectionMask(hEIO, &dirmask))
	{
		RETAILMSG(TRUE, (_T("failed (%i)\r\n"), GetLastError()));
		return -1;
	}
	RETAILMSG(TRUE, (_T("  0x%x.\r\n"), dirmask));

	// set the initial line states (all on)
	RETAILMSG(TRUE, (_T("  setting initial state to 0x%x..."), state));
	if(!EIOWrite(hEIO, state))
	{
		RETAILMSG(TRUE, (_T("failed (%i)\r\n"), GetLastError()));
		return -1;
	}
	RETAILMSG(TRUE, (_T("  ok.\r\n")));

	// get the initial line states (all on)
	RETAILMSG(TRUE, (_T("  reading back initial state...")));
	if(!EIORead(hEIO, &state))
	{
		RETAILMSG(TRUE, (_T("failed (%i)\r\n"), GetLastError()));
		return -1;
	}
	RETAILMSG(TRUE, (_T("  0x%x\r\n"), state));

	// start the test
	hThread = CreateThread(NULL, NULL, TestThread, hEIO, 0, &threadID);

	// run for five minutes, max
	for (int min_left = 5 ; min_left > 0 ; min_left--)
	{
		RETAILMSG (TRUE, (TEXT("\r\n---- %d minutes left in test ----\r\n\n"),min_left));
		Sleep(60*1000);
	}

	// kill the test thread
	bRun = FALSE;

	RETAILMSG(TRUE, (_T("-------- End EIOTest --------\r\n")));

	return 0;
}


ULONG TestThread(void *arg)
{
	HANDLE	hEIO = (HANDLE)arg;
	DWORD	readState = 0;
	TCHAR	readbinary[17];
	TCHAR	writebinary[17];
	int		m, i = 0;
	DWORD	msk;
	int		lastbit = 15;
	
	
	while(bRun)
	{
		// write a test pattern across the entire DWORD
		// this includes lines that are set as input and
		// non-existing lines to demostrate what happens
		// in each case
		state = 0;
		msk = 0;

		for(m = lastbit ; m >= 0 ; m--)
		{
			for(i = 0 ; i <= m ; i++)
			{
				state = (1<<i) | msk;

				// write the value to EIO
				if(EIOWrite(hEIO, state))
				{
					// read back the current state
					EIORead(hEIO, &readState);

					// display what we wrote and read
					ToBinary(state, writebinary);
					ToBinary(readState, readbinary);
					RETAILMSG(TRUE, (_T("W: %s    R: %s\r\n"), writebinary, readbinary));
				}
				Sleep(100);
			}
			msk |= (1<< (i-1));
		}

		for(m = 0 ; m <= lastbit ; m++)
		{
			for(i = 0 ; i <= m ; i++)
			{
				state = (1<<~i) | msk | (1<<(lastbit - (m-i)));

				// write the value to EIO
				if(EIOWrite(hEIO, state))
				{
					// read back the current state
					EIORead(hEIO, &readState);

					// display what we wrote and read
					ToBinary(state, writebinary);
					ToBinary(readState, readbinary);
					RETAILMSG(TRUE, (_T("W: %s    R: %s\r\n"), writebinary, readbinary));
				}				
				Sleep(100);
			}

			msk &= (~(1 << (lastbit - i + 1)));
		}
/*
		for( ; state > 0 ; state--)
		{
			// write the value to EIO
			if(EIOWrite(hEIO, state))
			{
				// read back the current state
				EIORead(hEIO, &readState);

				// display what we wrote and read
				ToBinary(state, writebinary);
				ToBinary(readState, readbinary);
				RETAILMSG(TRUE, (_T("W: %s    R: %s\r\n"), writebinary, readbinary));

				// sleep
				Sleep(100);
			}
		}		

		for( ; state < 0xFFFF ; state++)
		{
			// write the value to EIO
			if(EIOWrite(hEIO, state))
			{
				// read back the current state
				EIORead(hEIO, &readState);

				// display what we wrote and read
				ToBinary(state, writebinary);
				ToBinary(readState, readbinary);
				RETAILMSG(TRUE, (_T("W: %s    R: %s\r\n"), writebinary, readbinary));

				// sleep
				Sleep(100);
			}
		}
*/
	}
	return 0;
}

void ToBinary(DWORD number, TCHAR *result)
{
	_stprintf(result, _T("%i%i%i%i %i%i%i%i %i%i%i%i %i%i%i%i"), 
		(number>>0xF & 1), (number>>0xE & 1), (number>>0xD & 1), (number>>0xC & 1),
		(number>>0xB & 1), (number>>0xA & 1), (number>>0x9 & 1), (number>>0x8 & 1),
		(number>>0x7 & 1), (number>>0x6 & 1), (number>>0x5 & 1), (number>>0x4 & 1),
		(number>>0x3 & 1), (number>>0x2 & 1), (number>>0x1 & 1), (number & 1));
}

BOOL EIOSetDirectionMask(HANDLE hEIO, DWORD mask)
{
	BOOL	success = TRUE;
	DWORD	NumberOfBytesWritten;

	if(!DeviceIoControl(hEIO, 
					    IOCTL_EIO_CONFIG_PORTS,
						&mask, 
						sizeof(DWORD), 
						NULL, 
						0, 
						&NumberOfBytesWritten, 
						NULL))
	{
		RETAILMSG(TRUE, (_T("Failed to set EIO direction mask\r\nGetLastError() returned %i"), GetLastError()));
		success = FALSE;
	}

	return success;
}

BOOL EIOGetDirectionMask(HANDLE hEIO, DWORD *mask)
{
	BOOL	success = TRUE;
	DWORD	NumberOfBytesRead;

	if(!DeviceIoControl(hEIO, 
					    IOCTL_EIO_READ_CONFIG,
						NULL,
						0,
						&mask,
						sizeof(DWORD), 
						&NumberOfBytesRead, 
						NULL))
	{
		RETAILMSG(TRUE, (_T("Failed to get EIO direction mask\r\nGetLastError() returned %i"), GetLastError()));
		success = FALSE;
	}

	return success;
}

BOOL EIOWrite(HANDLE hEIO, DWORD state)
{
	BOOL	success					= TRUE;
	DWORD	NumberOfBytesWritten	= 0;

	// set the state
	if(!WriteFile(hEIO, &state, sizeof(DWORD), &NumberOfBytesWritten, NULL))
	{
		RETAILMSG(TRUE, (_T("Failed to Write to EIO\r\nGetLastError() returned %i"), GetLastError()));
		success = FALSE;
	}

	return success;
}

BOOL EIORead(HANDLE hEIO, DWORD *state)
{
	BOOL	success				= TRUE;
	DWORD   NumberOfBytesRead	= 0;

	// get the state
	if(!ReadFile(hEIO, state, sizeof(DWORD), &NumberOfBytesRead, NULL))
	{
		RETAILMSG(TRUE, (_T("Failed to Read from EIO\r\nGetLastError() returned %i"), GetLastError()));
		success = FALSE;
	}

	return success;
}